local LubStub = LibStub
local DeathClock = LibStub('AceAddon-3.0'):NewAddon('DeathClock', 'AceEvent-3.0', 'AceConsole-3.0')
_G.DeathClock = DeathClock
local SML = LibStub("LibSharedMedia-3.0")
local debug = false

local defaults = {
	global = {
		targetFrame = { 
			show = true,
			background = {r = 0,b = 0,g = 0},
			foreground = {r = 1,b = 1,g = 1},
			backgroundAlpha = .25,
			foregroundAlpha = 1,
			font = "DorisPP",
			size = 10,
			outline = 'OUTLINE',
		},
		
		focusFrame = {
			show = true,
			background = {r = 0,b = 0,g = 0},
			foreground = {r = 1,b = 1,g = 1},
			backgroundAlpha = .25,
			foregroundAlpha = 1,
			font = "DorisPP",
			size = 10,
			outline = 'OUTLINE',
		},
		
		bossFrame = {
			show = true,
			background = {r = 0,b = 0,g = 0},
			foreground = {r = 1,b = 1,g = 1},
			backgroundAlpha = .25,
			foregroundAlpha = 1,
			font = "DorisPP",
			size = 10,
			outline = 'OUTLINE',
		}
	}
}

local registerOnShow = function(frame) 
	frame.health0, frame.time0, frame.mhealth, frame.mtime = nil
	frame:RegisterEvent("UNIT_HEALTH") 
end

local unregisterOnHide = function(frame) 
	frame:UnregisterEvent("UNIT_HEALTH")
end

local startDrag = function(frame) 
	if IsAltKeyDown() then 
		frame:StartMoving() 
	end 
end

local stopDrag = function(frame) 
	frame:StopMovingOrSizing() 
end

local timetodieUpdate = function(frame, e, unit) 
	local health = UnitHealth(frame.target)
	local time = GetTime()

	if e == frame.resetEvent or health == UnitHealthMax(frame.target) then
		frame.health0, frame.time0, frame.mhealth, frame.mtime = nil
		frame.text:SetText("...")
		return
	end

	if not frame.health0 then
		frame.health0, frame.time0 = health, time
		frame.mhealth, frame.mtime = health, time
		return
	end

	frame.mhealth = (frame.mhealth + health) * .5
	frame.mtime = (frame.mtime + time) * .5

	if frame.mhealth > frame.health0 then
		frame.text:SetText("...")
		frame.health0, frame.time0, frame.mhealth, frame.mtime = nil
	else
		time = ceil(health * (frame.time0 - frame.mtime) / (frame.mhealth - frame.health0) )
		if time < 0 then
			frame.text:SetText("...")
			frame.health0, frame.time0, frame.mhealth, frame.mtime = nil
		elseif time < 60 then
			frame.text:SetText(format('%ds',time))
		elseif time < 3600 then
			frame.text:SetText(format('%dm%02ds', 1/60 * time, time % 60))
		elseif time < 86400 then
			frame.text:SetText(format('%dh%02dm', 1/3600 * time, 1/60 * (time % 3600)))
		else
			frame.text:SetText(format('%dd%02dh', 1/86400 * time, 1/3600 * (time % 86400)))
		end
	end
	
	if debug == true then
		print(frame:GetName(), frame.text:GetText())
	end
end

function DeathClock:SetupFrame(frameName,parent,textName,target,resetEvent)
	local frame = CreateFrame("Frame", frameName, parent)

	frame:SetPoint("LEFT", parent, "LEFT", -50, -12)
	frame:SetWidth(51)
	frame:SetHeight(16)
	frame.texture = frame:CreateTexture("bg", "BACKGROUND", "ActionBarButtonTemplate")
	frame.texture:SetAllPoints(frame)
	frame.texture:SetTexture(0,0,0,.8)

	frame.text = frame:CreateFontString(textName, "OVERLAY")
	frame.text:SetAllPoints(frame)
	frame.text:SetFontObject(TextStatusBarText)
	frame.text:SetJustifyH("RIGHT")
	if resetEvent then
		frame:RegisterEvent(resetEvent)
		frame.resetEvent = resetEvent
	else
		frame.resetEvent = "NoResetEvent"
	end
	frame:SetScript("OnShow", registerOnShow)
	frame:SetScript("OnHide", unregisterOnHide)
	frame:EnableMouse(true)
	frame:RegisterForDrag("LeftButton")
	frame:SetClampedToScreen(true)
	frame:SetMovable(true)
	frame:SetScript("OnDragStart", startDrag)
	frame:SetScript("OnDragStop", stopDrag)
	frame:SetScript("OnEvent", timetodieUpdate)
	frame.target = target
	return frame
end

function DeathClock:ApplyFrameSettings(frame,settings)
	frame.text:SetFont(SML:Fetch('font', settings.font), settings.size, settings.outline)
	frame.text:SetTextColor(settings.foreground.r, settings.foreground.g, settings.foreground.b, settings.foregroundAlpha)
	frame.texture:SetTexture(settings.background.r, settings.background.g, settings.background.b, settings.backgroundAlpha)
	if settings.show == true then
		frame:Show()
	else
		frame:Hide()
	end
end

function DeathClock:OnInitialize()
	local db = LibStub('AceDB-3.0'):New('DeathClockDB', defaults, 'Default')
	self.db = db
	self:RegisterOptions()
	
	self.t_dc =  self:SetupFrame("TargetDeathClock", TargetFrameHealthBar,      "TargetDeathClockText", "target", "PLAYER_TARGET_CHANGED")
	self.f_dc =  self:SetupFrame("FocusDeathClock",  FocusFrameHealthBar,       "FocusDeathClockText",  "focus",  "PLAYER_FOCUS_CHANGED")
	self.b1_dc = self:SetupFrame("Boss1DeathClock",  Boss1TargetFrameHealthBar, "Boss1DeathClockText",  "boss1",  nil)
	self.b2_dc = self:SetupFrame("Boss2DeathClock",  Boss2TargetFrameHealthBar, "Boss2DeathClockText",  "boss2",  nil)
	self.b3_dc = self:SetupFrame("Boss3DeathClock",  Boss3TargetFrameHealthBar, "Boss3DeathClockText",  "boss3",  nil)
	self.b4_dc = self:SetupFrame("Boss4DeathClock",  Boss4TargetFrameHealthBar, "Boss4DeathClockText",  "boss4",  nil)
	
	self:ApplySettings(self.db.global)
	self:RegisterChatCommand("dcdebug", "Debug")
end

function DeathClock:Debug()
	if debug == true then
		debug = false
	else
		debug = true
	end
end

local function GetFontList()
	return AceGUIWidgetLSMlists.font
end

local function GetValue(info)
	return info.handler.db.global[info.arg[1]][info.arg[2]]
end

local function SetValue(info, value)
	local self = info.handler
	local global = self.db.global
	global[info.arg[1]][info.arg[2]] = value
	self:ApplySettings(global)
end

local function GetColorValue(info)
	local color = info.handler.db.global[info.arg[1]][info.arg[2]]
	return color.r, color.g, color.b
end

local function SetColorValue(info, r, g, b)
	local self = info.handler
	local global = self.db.global
	global[info.arg[1]][info.arg[2]].r = r
	global[info.arg[1]][info.arg[2]].g = g
	global[info.arg[1]][info.arg[2]].b = b
	self:ApplySettings(global)
end

function DeathClock:ApplySettings(global)
	self:ApplyFrameSettings( self.t_dc, global.targetFrame)
	self:ApplyFrameSettings( self.f_dc, global.focusFrame)
	self:ApplyFrameSettings( self.b1_dc, global.bossFrame)
	self:ApplyFrameSettings( self.b2_dc, global.bossFrame)
	self:ApplyFrameSettings( self.b3_dc, global.bossFrame)
	self:ApplyFrameSettings( self.b4_dc, global.bossFrame)
end

local options = {
	name = 'Death Clock',
	type = 'group',
	desc = 'Display an estimated kill time for your target, focus target and any mobs in the blizzard boss frames.',
	set = SetValue,
	get = GetValue,
	handler = DeathClock,
	args = {
		["TARGET_FRAME"] = {
			name = "Target",
			type = 'group',
			order = 100,
			guiInline = true,
			set = SetValue,
			get = GetValue,
			disabled = IsFrameDisabled,
			args = {
				["Show"] = {
					name = "Show",
					type = 'toggle',
					order = 100,
					arg = {"targetFrame",'show'},
					disabled = false,
				},
				["Font Size"] = {
					name = "Font Size",
					type = 'range',
					min = 4, max = 20, step = 1,
					order = 150,
					arg = {"targetFrame",'size'},
				},
				["Font"] = {
					name = "Font",
					type = 'select',
					dialogControl = 'LSM30_Font',
					values = GetFontList,
					order = 200,
					arg = {"targetFrame",'font'},
				},
				["FColor"] = {
					name = "Text Color",
					type = 'color',
					set = SetColorValue,
					get = GetColorValue,
					order = 350,
					arg = {"targetFrame",'foreground'},
				},
				["TextAlpha"] = {
					name = "Text Alpha",
					type = 'range',
					min = 0, max = 1, step = .05,
					order = 400,
					arg = {"targetFrame",'foregroundAlpha'},
				},
				["Outline"] = {
					name = "Outline",
					type = 'select',
					desc = "Set font outline.",
					values = {[''] = NONE, OUTLINE = VOICE_CHAT_NORMAL, THICKOUTLINE = "Thick"},
					order = 425,
					arg = {"targetFrame",'outline'},
				},
				["BColor"] = {
					name = "Background",
					type = 'color',
					set = SetColorValue,
					get = GetColorValue,
					order = 450,
					arg = {"targetFrame",'background'},
				},
				["BAlpha"] = {
					name = "Background Alpha",
					type = 'range',
					min = 0, max = 1, step = .05,
					order = 500,
					arg = {"targetFrame",'backgroundAlpha'},
				},
			}
		},
	
		["FOCUS_FRAME"] = {
			name = "Focus",
			type = 'group',
			order = 150,
			guiInline = true,
			set = SetValue,
			get = GetValue,
			disabled = IsFrameDisabled,
			args = {
				["Show"] = {
					name = "Show",
					type = 'toggle',
					order = 100,
					arg = {"focusFrame",'show'},
					disabled = false,
				},
				["Font Size"] = {
					name = "Font Size",
					type = 'range',
					min = 4, max = 20, step = 1,
					order = 150,
					arg = {"focusFrame",'size'},
				},
				["Font"] = {
					name = "Font",
					type = 'select',
					dialogControl = 'LSM30_Font',
					values = GetFontList,
					order = 200,
					arg = {"focusFrame",'font'},
				},
				["FColor"] = {
					name = "Text Color",
					type = 'color',
					set = SetColorValue,
					get = GetColorValue,
					order = 350,
					arg = {"focusFrame",'foreground'},
				},
				["TextAlpha"] = {
					name = "Text Alpha",
					type = 'range',
					min = 0, max = 1, step = .05,
					order = 400,
					arg = {"focusFrame",'foregroundAlpha'},
				},
				["Outline"] = {
					name = "Outline",
					type = 'select',
					desc = "Set font outline.",
					values = {[''] = NONE, OUTLINE = VOICE_CHAT_NORMAL, THICKOUTLINE = "Thick"},
					order = 425,
					arg = {"focusFrame",'outline'},
				},
				["BColor"] = {
					name = "Background",
					type = 'color',
					set = SetColorValue,
					get = GetColorValue,
					order = 450,
					arg = {"focusFrame",'background'},
				},
				["BAlpha"] = {
					name = "Background Alpha",
					type = 'range',
					min = 0, max = 1, step = .05,
					order = 500,
					arg = {"focusFrame",'backgroundAlpha'},
				},
			}
		},	
		
		["BOSS_FRAME"] = {
			name = "Boss",
			type = 'group',
			order = 200,
			guiInline = true,
			set = SetValue,
			get = GetValue,
			disabled = IsFrameDisabled,
			args = {
				["Show"] = {
					name = "Show",
					type = 'toggle',
					order = 100,
					arg = {"bossFrame",'show'},
					disabled = false,
				},
				["Font Size"] = {
					name = "Font Size",
					type = 'range',
					min = 4, max = 20, step = 1,
					order = 150,
					arg = {"bossFrame",'size'},
				},
				["Font"] = {
					name = "Font",
					type = 'select',
					dialogControl = 'LSM30_Font',
					values = GetFontList,
					order = 200,
					arg = {"bossFrame",'font'},
				},
				["Outline"] = {
					name = "Outline",
					type = 'select',
					desc = "Set font outline.",
					values = {[''] = NONE, OUTLINE = VOICE_CHAT_NORMAL, THICKOUTLINE = "Thick"},
					order = 300,
					arg = {"bossFrame",'outline'},
				},
				["FColor"] = {
					name = "Text Color",
					type = 'color',
					set = SetColorValue,
					get = GetColorValue,
					order = 350,
					arg = {"bossFrame",'foreground'},
				},
				["TextAlpha"] = {
					name = "Text Alpha",
					type = 'range',
					min = 0, max = 1, step = .05,
					order = 400,
					arg = {"bossFrame",'foregroundAlpha'},
				},
				["Outline"] = {
					name = "Outline",
					type = 'select',
					desc = "Set font outline.",
					values = {[''] = NONE, OUTLINE = VOICE_CHAT_NORMAL, THICKOUTLINE = "Thick"},
					order = 425,
					arg = {"bossFrame",'outline'},
				},
				["BColor"] = {
					name = "Background",
					type = 'color',
					set = SetColorValue,
					get = GetColorValue,
					order = 450,
					arg = {"bossFrame",'background'},
				},
				["BAlpha"] = {
					name = "Background Alpha",
					type = 'range',
					min = 0, max = 1, step = .05,
					order = 500,
					arg = {"bossFrame",'backgroundAlpha'},
				},
			}
		}	
	},
}

function DeathClock:RegisterOptions()
	local global = LibStub('AceDBOptions-3.0'):GetOptionsTable(self.db)
	local registry = LibStub('AceConfigRegistry-3.0')
	local dialog = LibStub('AceConfigDialog-3.0')
	LibStub('AceConfig-3.0'):RegisterOptionsTable('Death Clock', options, {'Death Clock', 'Death Clock'})
	syncFrameCount(getFrameCount())
	registry:RegisterOptionsTable('Death Clock Options', options)

	local main = dialog:AddToBlizOptions('Death Clock Options', 'Death Clock')

	
	local dataobj = LibStub:GetLibrary('LibDataBroker-1.1'):NewDataObject('Death Clock', {
		type = 'data source',
		text = 'Death Clock',
		label = 'Death Clock',
		icon = 'Interface\\Icons\\Spell_Holy_BorrowedTime',
	})
	
	if dataobj then
		dataobj.OnClick = function() InterfaceOptionsFrame_OpenToCategory(main) end
		dataobj.OnTooltipShow = function(tooltip)
			tooltip:AddLine('Death Clock')
			tooltip:AddLine('Display an estimated kill time for your target, focus target and any mobs in the blizzard boss frames.')
		end
	end
end

